=== modified file 'src/lazr/restful/NEWS.txt'
--- src/lazr/restful/NEWS.txt	2010-10-25 15:17:49 +0000
+++ src/lazr/restful/NEWS.txt	2010-10-25 16:31:09 +0000
@@ -5,7 +5,8 @@
 0.14.1 (2010-10-24)
 ===================
 
-Fixed some encoding errors when error strings contain non-ASCII characters.
+Fixed a unicode encoding bug that precluded reporting exceptions with
+non-ASCII characters.
 
 0.14.0 (2010-10-05)
 ===================

=== modified file 'src/lazr/restful/example/base/tests/entry.txt'
--- src/lazr/restful/example/base/tests/entry.txt	2010-10-25 14:55:37 +0000
+++ src/lazr/restful/example/base/tests/entry.txt	2010-10-25 16:31:09 +0000
@@ -445,17 +445,6 @@
     HTTP/1.1 209 Content Returned
     ...
 
-Miscellaneous error checking
-============================
-
-A validation error that contains non-ASCII characters will be
-formatted properly.
-
-    >>> error = modify_cookbook('Everyday Greens',
-    ...     {'cuisine' : 'No such cuisine'}, 'PATCH')
-    >>> error.body
-    'cuisine: Invalid value "No such cuisine". Acceptable values are... Fran\xc3\xa7aise'
-
 Content negotiation during modification
 =======================================
 

=== modified file 'src/lazr/restful/tests/test_webservice.py'
--- src/lazr/restful/tests/test_webservice.py	2010-08-25 13:21:39 +0000
+++ src/lazr/restful/tests/test_webservice.py	2010-10-25 16:31:09 +0000
@@ -15,10 +15,11 @@
 from zope.interface import implements, Interface
 from zope.interface.interface import InterfaceClass
 from zope.publisher.browser import TestRequest
-from zope.schema import Date, Datetime, TextLine
+from zope.schema import Choice, Date, Datetime, TextLine
 from zope.security.management import newInteraction, queryInteraction
 from zope.traversing.browser.interfaces import IAbsoluteURL
 
+from lazr.enum import EnumeratedType, Item
 from lazr.restful import ResourceOperation
 from lazr.restful.fields import Reference
 from lazr.restful.interfaces import (
@@ -158,8 +159,9 @@
         request = getUtility(IWebServiceConfiguration).createRequest(
             StringIO(""), {})
 
-        entry = entry_class(HasRestrictedField(""), request)
-        resource = EntryResource(HasRestrictedField(""), request)
+        data_object = HasRestrictedField("")
+        entry = entry_class(data_object, request)
+        resource = EntryResource(data_object, request)
         entry.schema['a_field'].restrict_to_interface = IHasRestrictedField
         self.assertEquals(resource.entry.a_field, '')
         resource.applyChanges({'a_field': u'a_value'})
@@ -174,6 +176,62 @@
                           {'a_field': u'a_new_value'})
         self.assertEquals(resource.entry.a_field, 'a_value')
 
+
+class UnicodeChoice(EnumeratedType):
+    """A choice between an ASCII value and a Unicode value."""
+    ASCII = Item("Ascii", "Ascii choice")
+    UNICODE = Item(u"Uni\u00e7ode", "Uni\u00e7ode choice")
+
+
+class ICanBeSetToUnicodeValue(Interface):
+    """An entry with an InterfaceRestrictedField."""
+    export_as_webservice_entry()
+    a_field = exported(Choice(
+        vocabulary=UnicodeChoice,
+        title=u"A value that might be ASCII or Unicode.",
+        required=False, default=None))
+
+
+class CanBeSetToUnicodeValue:
+    """An implementation of ICanBeSetToUnicodeValue."""
+    implements(ICanBeSetToUnicodeValue)
+    def __init__(self, value):
+        self.a_field = value
+
+
+class UnicodeErrorTestCase(WebServiceTestCase):
+    """Test that Unicode error strings are properly passed through."""
+
+    testmodule_objects = [CanBeSetToUnicodeValue, ICanBeSetToUnicodeValue]
+
+    def setUp(self):
+        super(UnicodeErrorTestCase, self).setUp()
+        getGlobalSiteManager().registerAdapter(
+            DummyAbsoluteURL,
+            [ICanBeSetToUnicodeValue, IWebServiceClientRequest], IAbsoluteURL)
+
+    def test_unicode_error(self):
+        entry_class = get_resource_factory(ICanBeSetToUnicodeValue, IEntry)
+        request = getUtility(IWebServiceConfiguration).createRequest(
+            StringIO(""), {})
+
+        data_object = CanBeSetToUnicodeValue("")
+        entry = entry_class(data_object, request)
+        resource = EntryResource(data_object, request)
+
+        # This will raise an exception, which will cause the request
+        # to fail with a 400 error code.
+        error = resource.applyChanges({'a_field': u'No such value'})
+        self.assertEqual(request.response.getStatus(), 400)
+
+        # The error message is a Unicode string which mentions both
+        # the ASCII value and the Unicode value,
+        expected_error = (
+            u'a_field: Invalid value "No such value". Acceptable values '
+            u'are: Ascii, Uni\u00e7ode')
+        self.assertEquals(error, expected_error)
+
+
 class WadlAPITestCase(WebServiceTestCase):
     """Test the docstring generation."""
 

